diff -ur ../sendmail-8.13.8-orig/cf/m4/proto.m4 ././cf/m4/proto.m4
--- ../sendmail-8.13.8-orig/cf/m4/proto.m4	2006-03-30 15:50:13.000000000 -0500
+++ ././cf/m4/proto.m4	2006-11-08 21:55:14.000000000 -0500
@@ -441,6 +441,8 @@
 _OPTION(Timeout.lhlo, `confTO_LHLO', `2m')
 _OPTION(Timeout.auth, `confTO_AUTH', `10m')
 _OPTION(Timeout.starttls, `confTO_STARTTLS', `1h')
+ifdef(`_FFR_TIMEOUT_FLUSH_RESPONSE', `dnl
+_OPTION(Timeout.flushresponse, `confTO_TIMEOUT_FLUSH_RESPONSE', `30s')')
 
 # time for DeliverBy; extension disabled if less than 0
 _OPTION(DeliverByMin, `confDELIVER_BY_MIN', `0')
diff -ur ../sendmail-8.13.8-orig/sendmail/conf.c ././sendmail/conf.c
--- ../sendmail-8.13.8-orig/sendmail/conf.c	2006-03-22 17:49:33.000000000 -0500
+++ ././sendmail/conf.c	2006-11-08 21:52:16.000000000 -0500
@@ -6262,6 +6262,11 @@
 	/* Disable PIPELINING, delay client if used. */
 	"_FFR_NO_PIPE",
 #endif /* _FFR_NO_PIPE */
+#if _FFR_TIMEOUT_FLUSH_RESPONSE
+	/* flush response to client if enough time has passed, regardless
+	 * of PIPELINE conditions */
+	"_FFR_TIMEOUT_FLUSH_RESPONSE",
+#endif /* _FFR_TIMEOUT_FLUSH_RESPONSE */
 #if _FFR_LOG_NTRIES
 	/* log ntries=, from Nik Clayton of FreeBSD */
 	"_FFR_LOG_NTRIES",
diff -ur ../sendmail-8.13.8-orig/sendmail/readcf.c ././sendmail/readcf.c
--- ../sendmail-8.13.8-orig/sendmail/readcf.c	2006-03-02 14:17:09.000000000 -0500
+++ ././sendmail/readcf.c	2006-11-08 21:53:16.000000000 -0500
@@ -4136,6 +4136,10 @@
 	{ "queuewarn.dsn",		TO_QUEUEWARN_DSN		},
 #define TO_QUEUERETURN_DSN		0x25
 	{ "queuereturn.dsn",		TO_QUEUERETURN_DSN		},
+#if _FFR_TIMEOUT_FLUSH_RESPONSE
+#define TO_TIMEOUT_FLUSH_RESPONSE		0x26
+	{ "flushresponse",		TO_TIMEOUT_FLUSH_RESPONSE	},
+#endif /* _FFR_TIMEOUT_FLUSH_RESPONSE */
 	{ NULL,				0				},
 };
 
@@ -4363,6 +4367,12 @@
 		break;
 #endif /* STARTTLS */
 
+#if _FFR_TIMEOUT_FLUSH_RESPONSE
+	  case TO_TIMEOUT_FLUSH_RESPONSE:
+	  	TimeOuts.to_flush_response = convtime(val, 's');
+		break;
+#endif /* _FFR_TIMEOUT_FLUSH_RESPONSE */
+
 	  default:
 		syserr("settimeout: invalid timeout %s", name);
 		break;
diff -ur ../sendmail-8.13.8-orig/sendmail/sendmail.h ././sendmail/sendmail.h
--- ../sendmail-8.13.8-orig/sendmail/sendmail.h	2006-05-22 21:32:07.000000000 -0400
+++ ././sendmail/sendmail.h	2006-11-08 21:54:37.000000000 -0500
@@ -2018,6 +2018,9 @@
 #if STARTTLS
 	time_t	to_starttls;	/* STARTTLS dialogue [10m] */
 #endif /* STARTTLS */
+#if _FFR_TIMEOUT_FLUSH_RESPONSE
+	time_t	to_flush_response; /* how long until we force response flushing */
+#endif /* _FFR_TIMEOUT_FLUSH_RESPONSE */
 			/* following are per message */
 	time_t	to_q_return[MAXTOCLASS];	/* queue return timeouts */
 	time_t	to_q_warning[MAXTOCLASS];	/* queue warning timeouts */
diff -ur ../sendmail-8.13.8-orig/sendmail/srvrsmtp.c ././sendmail/srvrsmtp.c
--- ../sendmail-8.13.8-orig/sendmail/srvrsmtp.c	2006-07-07 12:29:39.000000000 -0400
+++ ././sendmail/srvrsmtp.c	2006-11-09 08:13:59.000000000 -0500
@@ -1062,6 +1062,23 @@
 		FileName = NULL;
 		(void) sm_io_flush(smioout, SM_TIME_DEFAULT);
 
+#if PIPELINING && _FFR_TIMEOUT_FLUSH_RESPONSE 
+		if (TimeOuts.to_flush_response > -1)
+		{
+			static time_t last_flush = 0;
+			time_t now_time = curtime();
+
+			if (!last_flush)
+				last_flush = now_time;
+			else if (now_time - last_flush > TimeOuts.to_flush_response)
+			{
+				(void) sm_io_flush(OutChannel, SM_TIME_DEFAULT);
+				last_flush = now_time;
+			}
+		}
+#endif /* PIPELINING && _FFR_TIMEOUT_FLUSH_RESPONSE */
+
+
 		/* read the input line */
 		SmtpPhase = "server cmd read";
 		sm_setproctitle(true, e, "server %s cmd read", CurSmtpClient);
